﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.IO.Ports;
using SimpleGraphicsLib;

namespace RunnerGame_CSharp
{
	public partial class MainWindow : Form
	{
		private static SerialPort SerialPort;

		public static int Gravity { get; set; }			// Amount of downward acceleration to apply per frame
		public int MinRight { get; set; }				// Account for jumpy analog input
		public int MinLeft { get; set; }				// Minimum left value to actually register movement
		public static Platform StartingPlatform { get; set; }  // Main platform
		public static List<Platform> Platforms { get; set; }   // Holds all of the game platforms
		public static Character Player { get; set; }	// The Player object
		public static bool GameOver { get; set; }		// To decide if "Game Over" should be displayed
		public static int height { get; set; }			// Sets the window's height
		public static int width { get; set; }			// Window width
		public static Panel Disp { get; set; }			// The Panel object to display graphics in
		private Font DrawFont { get; set; }				// Font to display text
		private SolidBrush FontBrush { get; set; }		// Also used to display text
		private static int minPlatformSpacing = 50;     // Minimum space between platforms
		private static int maxPlatformSpacing = 250;    // Minimum space between platforms
		private static int nextX;                       // Start first block at edge of screen
		private static int score = 0;                   // Track the current score
		private static Random random = new Random();
		private int NumberOfPlatforms = 10;

		public MainWindow()
		{
			InitializeComponent();

			string portName = "";
			try
			{
				portName = SerialPort.GetPortNames()[ 1 ];	// Set this to your COM port

			} catch ( Exception ) { }

			height = 700;								// Initialize all of the main variables...
			width = 1000;
			this.Width = width;
			this.Height = height;
			DisplayBox.Width = width - 16;
			DisplayBox.Height = height - 38;
			Gravity = 4;
			MinRight = 520;
			MinLeft = 500;
			GameOver = false;
			StartingPlatform = new Platform( 0 );       // Instantiate the starting platform
			Disp = DisplayBox;

			foreach( string port in SerialPort.GetPortNames() )
				Console.WriteLine( "AVAILABLE PORTS: " + port );
			
			if ( !String.IsNullOrEmpty( portName ) )
			{
														//portName, baud, parity, data bits, stop bit
				SerialPort = new SerialPort( portName, 9600, Parity.None, 8, StopBits.One );
				SerialPort.DataReceived +=				// Attach the handler method
				  new SerialDataReceivedEventHandler( DataReceived );
				SerialPort.Open();                      // Begin communications

			} // if !null

			
			Platforms = new List<Platform>( 10 );       // Create platforms
			StartingPlatform = new Platform( 0 );       // Create starting platform

			nextX = Width;								// Calculate X's based on previous one
			for ( int i = 0; i < NumberOfPlatforms; i++ ) // Only instantiate n-1 platforms 
			{
				Platforms.Add( new Platform( nextX ) );	// Build our List of platforms
				
				Platforms[ i ].Y = random.Next( height - height / 2 - Platforms[ i ].Height,
					height - height / 4 - Platforms[ i ].Height );
				Platforms[ i ].BoxColor = Color.FromArgb( random.Next( 0, 200 ),
					random.Next( 0, 200 ), random.Next( 0, 200 ) );
				Platforms[i].Width = random.Next( 100, 500 );

				GetNextX( Platforms[ i ] );             // Make sure x values don't overlap

			} // for

			Platforms.Add( StartingPlatform );			// StartingPlatform is the last platform in the List

			Player = new Character();					// Create the player object

			Reset();									// Set to the initial game state

		} // Constructor


		/// <summary>
		/// Resets the game to the original state.
		/// </summary>
		public static void Reset()
		{
			score = 0;
			StartingPlatform.Y = height - height / 4 + Player.Height + 1;
			StartingPlatform.Height = 512;
			StartingPlatform.Width = width;
			StartingPlatform.Scored = true;

			Player.Reset();

			GameOver = false;

		} // reset
		

		/// <summary>
		/// Gets a random X coordinate for a platform based on the previous platform's 
		///		X-coordinate and width, minimum spacing, and maximum spacing.
		/// </summary>
		/// <param name="platform"></param>
		public static void GetNextX( Platform platform )
		{
			// Increment minimum x by platform width + minimum platform spacing
			nextX = platform.X + platform.Width + random.Next( minPlatformSpacing, maxPlatformSpacing );

		}
		

		/// <summary>
		/// Used by the SerialPort class as a handler for data received on the Serial port.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private static void DataReceived( object sender, SerialDataReceivedEventArgs e )
		{
			String inString = SerialPort.ReadLine().Trim();     // Reads until '\n' reached

			// Show all the incoming data in the port's buffer:
			//Console.WriteLine( "Value Read: " + inString );   // DEBUG

			if ( !GameOver )
			{
				string[] strVals = inString.Split( ',' );			// Split the data into usable chunks
				float[] values = new float[strVals.Length];			// Holds converted float values
				for ( int i = 0; i < strVals.Length; i++ )
				{
					float.TryParse( strVals[i], out values[ i ] );  // Convert each value to a float
				}

				if ( values.Length >= 3 )				// x,y,game speed
				{
					// Use either of the next two lines depending on the behavior desired:
					Player.XSpeed = (int) values[ 2 ].Map( 0, 1023, 1, 30 );		// Set constant speed
					//Player.XSpeed = (int) values[ 0 ].Map( 0, 1023, -30, 30 );	// X-axis controls speed

					MovePlatforms();					// Pass player.xSpeed to the platforms

					Player.Collided = Player.CheckForCollisions( Platforms );

					if ( Player.Collided )              // Apply either a platform...
					{
						Player.ApplySurface( Character.CollidedPlatform.Y );
						if ( !Character.CollidedPlatform.Scored )
						{
							score++;
							Character.CollidedPlatform.Scored = true;
						}

					}
					else								// ... or gravity
					{
						Player.ApplyGravity( Gravity );

					} // Grounded/Not Grounded

					if ( Player.Y > height )            // If player has fallen into the abyss...
					{
						GameOver = true;                //	...stop all game activity
					}

				} // if values.length >= 4
				else if ( inString.Equals( "Jump" ) )
				{
					Player.Jump();						// Handle the Jump button action

				} // if values > 4

			}
			else if ( inString.Equals( "Jump" ) )       // gameOver == true
			{
				//println("GAME OVER - jump pressed...");
				Reset();

			} // if !GameOver else if "Jump"

			// "Invalidate" the panel to make it refresh the graphics
			Disp.Invalidate();

		} // DataReceived

		
		public void DisplayGameOver( PaintEventArgs e )
		{
			// Create font and brush.
			DrawFont = new Font( "Georgia", 48, FontStyle.Bold );
			FontBrush = new SolidBrush( Color.White );
			e.Graphics.DrawString( "Game Over", DrawFont, FontBrush,
				width / 2 - 180, height / 2 - 50 );

		} // DisplayGameOver


		public void DisplayScore( PaintEventArgs e )
		{
			// Create font and brush.
			DrawFont = new Font( "Tahoma", 17, FontStyle.Regular );
			FontBrush = new SolidBrush( Color.White );
			e.Graphics.DrawString( "Score: " + score.ToString(), DrawFont, FontBrush, 20, 20);

		} // DisplayScore


		public static void MovePlatforms()
		{
			if ( StartingPlatform.X >= -StartingPlatform.Width )
			{
				StartingPlatform.X = StartingPlatform.X - Player.XSpeed;

			} // startingPlatform

			for ( int i = 0; i < Platforms.Count - 1; i++ )
			{
				if ( Platforms[ i ].X < -Platforms[ i ].Width )		// If platform has gone offscreen
				{
					Platforms[ i ].X = nextX;                       // Move it to other side of screen
					Platforms[ i ].ResetRandom( random );
				}
				else
					Platforms[ i ].X = Platforms[ i ].X - Player.XSpeed; // Otherwise, just move it left
				
				GetNextX( Platforms[ i ] );

			} // for

			//Console.WriteLine( Platforms[ 0 ].ToString() );		// DEBUG

		} // MovePlatforms

		
		/// <summary>
		/// This handles the Paint event for the DisplayBox Windows Forms object. Each 
		/// time it needs to be refreshed, it draws all of the objects in the Panel.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		public void DisplayBox_OnPaint( object sender, PaintEventArgs e )
		{
			if ( StartingPlatform.X >= -StartingPlatform.Width )
			{
				StartingPlatform.OnPaint( e.Graphics );
			}

			DisplayBox.BackColor = Color.FromArgb( 66, 66, 150 );		// Window background
			Player.OnPaint( e.Graphics );           // Draw the player

			foreach ( Platform p in Platforms )     // Draw all the platforms
			{
				p.OnPaint( e.Graphics );

			} // for

			if( GameOver )							// Only show GameOver when var set
			{
				DisplayGameOver( e );
			}

			DisplayScore( e );                      // Draw score on top layer

		} // draw


	} // Class

} // Namespace
