﻿/*****************************************************************************************
* File Name:	Platform.cs																 *
* Programmer:   J.T. Blevins (jt.blevins@gmail.com)										 *
* Date:         11/25/2017                                                               *
* Purpose:      A class usable as a player or character. It extends the box class for	 *
*				use in a platformer game.												 *
*****************************************************************************************/

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using SimpleGraphicsLib;
using System.Drawing;

namespace RunnerGame_CSharp
{
	/// <summary>
	/// A class usable as a player or character. It extends the box class for 
	/// use in a platformer game.
	/// </summary>
	/// <seealso cref="SimpleGraphicsLib.Box" />
	public class Character : Box
	{
		public bool Jumping { get; set; }       // When to apply jump physics
		public bool Falling { get; set; }		// When to apply physics to handle falling
		public bool Collided { get; set; }		// True if collided with a Box
		public int XSpeed { get; set; }			// Affected by user input (Y-axis)
		public int YSpeed { get; set; }         // Affected by jumping
		public float JumpTimer { get; set; }    // The longer jumped, the more acceleration applied
		public float FallTimer { get; set; }    // The longer falling, the more acceleration applied
		public int JumpSpeed { get; set; }      // Initial value applied to Y-velocity when jumped
		public static Platform CollidedPlatform { get; set; }


		public Character()
		{
			Reset();

		} // Constructor


		/// <summary>
		/// Resets this instance.
		/// </summary>
		public void Reset()
		{
			Jumping = false;
			Falling = false;
			YSpeed = 0;
			XSpeed = 0;
			JumpSpeed = -90;
			JumpTimer = 0;
			FallTimer = 0;
			Height = 70;
			Width = 70;
			R = 230;
			G = 30;
			B = 30;
			BoxColor = Color.FromArgb( R, G, B );
			X = MainWindow.width / 2 - Width / 2;
			Y = MainWindow.height - MainWindow.height / 4 - Height / 2;
		}


		/// <summary>
		/// Applies the gravity.
		/// </summary>
		/// <param name="gravity">The gravity.</param>
		public void ApplyGravity( float gravity )
		{
			YSpeed += (int) ( FallTimer * gravity );    // Apply gravity until hit ground.
			Y += YSpeed;
			FallTimer++;

		} // ApplyGravity


		/// <summary>
		/// Applies the surface.
		/// </summary>
		/// <param name="platformTop">The platform top.</param>
		public void ApplySurface( int platformTop )
		{
			// Protect from going through the bottom
			if ( Y + YSpeed > platformTop )//(Y >= platformTop ||  //&& ( jumped || falling ) ) // break from gravity
			{
				Y = platformTop - Height;
				JumpTimer = 0;
				FallTimer = 0;
				Jumping = false;
				Falling = false;

			} // If player moves past a platform

		} // ApplySurface


		/// <summary>
		/// Checks the collided.
		/// </summary>
		/// <param name="otherObject">The platform.</param>
		/// <returns></returns>
		public override bool IsCollided( Box otherObject )
		{
			// If player and platform intersect, return true
			return IsCollidedX( otherObject )
			  && IsCollidedY( otherObject );

		} // CheckCollided


		/// <summary>
		/// Checks the collided x.
		/// </summary>
		/// <param name="otherObject">The platform.</param>
		/// <returns></returns>
		public override bool IsCollidedX( Box otherObject )
		{
			// If this x and platform x intersect, return true
			return ( X >= otherObject.X && X <= otherObject.X + otherObject.Width )
			  || ( X + Width >= otherObject.X && X + Width <= otherObject.X + otherObject.Width );

		} // checkCollidedX


		/// <summary>
		/// Checks the collided y.
		/// </summary>
		/// <param name="otherObject">The platform.</param>
		/// <returns></returns>
		public override bool IsCollidedY( Box otherObject )
		{
			if ( Jumping && YSpeed < 0 )				// Allow jumping through bottoms of platforms
				return false;
			else if ( Y + YSpeed >= otherObject.Y )		// If we'll be falling through a platform...
				return true;                            //		...just set collided (and land the player)

			// If this y and platform y intersect, return true
			return ( otherObject.Y <= Y + Height );
			  //|| ( otherObject.Y + otherObject.Height >= Y 
					//&& otherObject.Y + otherObject.Height <= Y + Height );

		} // checkCollidedY


		/// <summary>
		/// Checks for collisions.
		/// </summary>
		/// <param name="boxes">The boxes.</param>
		/// <returns></returns>
		public bool CheckForCollisions( List<Platform> boxes )
		{
			foreach ( Platform platform in boxes )
			{
				if ( IsCollided( platform ) )
				{
					CollidedPlatform = platform;
					return true;
				}
			}

			CollidedPlatform = null;
			return false;

		} // CheckForCollisions


		/// <summary>
		/// Jumps this Player.
		/// </summary>
		public void Jump()
		{
			if ( JumpTimer == 0 && !Jumping && !Falling )        // If timer is 0 and jumping...
			{
				YSpeed = JumpSpeed;               //   ...start jump
				Jumping = true;
			}

		} // Jump

		//} // ApplyGravity

	} // Class

} // Namespace
